//Maemo Barcode Reader and Interpreter (mbarcode or maemo-barcode)
//Copyright (C) 2010 Simon Pickering
//Copyright (C) 2010 Svenn-Arne Dragly
//
//Some source code obtained from other individuals/companies (not affiliated with the project), such as
//Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
//
//Various parts of barcode recognition and GStreamer manipulation code written by:
//      Timothy Terriberry
//      Adam Harwell
//      Jonas Hurrelmann
//
//Original GStreamer code based on the maemo-examples package:
//Copyright (C) 2007-2008 Nokia Corporation. All rights reserved.
//Copyright (C) 2006 INdT.
//@author Talita Menezes <talita.menezes@indt.org.br>
//@author Cidorvan Leite <cidorvan.leite@indt.org.br>
//@author Jami Pekkanen <jami.pekkanen@nokia.com>
//
//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.

// qt includes
#include <QtGui>
// local includes
#include "resultswindow.h"
#include "ui_resultswindow.h"
#include "pluginaction.h"
#include "plugininterfaces.h"
#include "pluginintermediate.h"
#include <QTimer>
#include <QItemSelectionModel>
#include "maemobarcodewindow.h"
#include <QMaemo5InformationBox>
#include "videowidget.h"

#define PLUGIN_COUNT_LIMIT    6

ResultsWindow::ResultsWindow(MaemoBarcodeWindow *parent) :
        QMainWindow(parent),
        ui(new Ui::ResultsWindow)
{
    ui->setupUi(this);
    this->parent = parent;

    setAttribute(Qt::WA_Maemo5StackedWindow);
    ui->tblResults->setItemDelegate(new PluginActionListDelegate(this));
    ui->tblResults->horizontalHeader()->setResizeMode(QHeaderView::Stretch);

    selectionTimer = new QTimer(this);
    connect(selectionTimer, SIGNAL(timeout()), this, SLOT(selectionTimerExpiry()));

    pluginUpdateCounter = 0;
}

ResultsWindow::~ResultsWindow()
{
    delete ui;
}

void ResultsWindow::setPluginActions(QList<PluginAction *>plugins) {
    qDebug() << "ResultsWindow::setPluginActions(): Called.";
    this->plugins = plugins;

    foreach(PluginAction* plugin, plugins) {
        // skip plugins that are not enabled
        if(plugin->getEnabled()) {
            qDebug() << "ResultsWindow::setPluginActions(): Checking action plugin " << plugin->getName();
            connect(plugin, SIGNAL(isReady()), this, SLOT(pluginReady()));
        }
    }

}

void ResultsWindow::hideEvent(QHideEvent *) {
    pluginUpdateCounter = 0;
    addedPlugins.clear(); // clear the plugins from the list
}

void ResultsWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void ResultsWindow::processBarcode(QString barcode) {
    if(barcode.length() > 40) {
        ui->lblResults->setText(tr("Results for '%1...'").arg(barcode.left(40)).simplified());
    } else {
        ui->lblResults->setText(tr("Results for '%1'").arg(barcode).simplified());
    }

    //pluginUpdateCounter = 0;
    //hideMeTimer->stop();

    //addedPlugins.clear(); // clear the plugins from the list

    // save the barcode text
    this->barcode_text = QString(barcode);

}

void ResultsWindow::pluginReady() {
    qDebug() << "ResultsWindow::pluginReady(): Called";
    bool addedNewPlugins=false;

    PluginAction* pluginAction = (PluginAction*)sender(); // find the plugin that sent the signal
    if(pluginAction) { // make sure the cast went ok
        if(!addedPlugins.contains(pluginAction)) { // don't add the plugin more than once
            addedPlugins.append(pluginAction);
            addedNewPlugins = true;
            pluginUpdateCounter++;
        }
    }

    if(addedNewPlugins==true){
        QAbstractItemModel *oldModel = ui->tblResults->model();
        PluginActionListModel *model = new PluginActionListModel(this, addedPlugins);
        ui->tblResults->setModel(model);
        delete(oldModel);
        ui->tblResults->resizeRowsToContents();
        ui->tblResults->setFixedHeight(ui->tblResults->verticalHeader()->length() + 60); // fix to make all contents of scrollwidget scroll together
    }

    if(addedPlugins.length()>0  && parent->AutoUseFirstPlugin){ // && this->parent->NumEnabledPlugins==1
        qDebug() << "AutoUseFirstPlugin, running this one";

        QMaemo5InformationBox::information (this->parent, addedPlugins.at(0)->getName() + " run", 1000);
        addedPlugins.at(0)->clickAction(this); // run the plugin
        //hideMeTimer->singleShot(1500, this, SLOT(hideMeTimerExpiry()));
        //this->close(); // hide this window
        //this->parent->activateWindow(); // send it a signal so that it starts the camera again
        //return;
        this->parent->videoWidget->start(false);
    }


}

void ResultsWindow::on_pushButton_clicked()
{
    QApplication::clipboard()->setText(this->barcode_text, QClipboard::Clipboard);
}

QSize PluginActionListDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) {
    Q_UNUSED(index)
    Q_UNUSED(option)
    return QSize();
}

void PluginActionListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                                     const QModelIndex &index) const {
    QStyledItemDelegate::paint(painter, option, index);

    PluginAction *e = qVariantValue<PluginAction *>(index.data());

    QFont font = option.font;
    QRect rect = option.rect;
    rect.adjust(65, 10, -20, -7);
    QRect irect = option.rect;
    irect.adjust(3, 10, -20, -7);

    painter->save();

    if(!e->getImage().isNull()) {
        painter->drawImage(QRect(irect.left(), irect.top(), 48, 48), e->getImage());
    }
    painter->drawText(rect, Qt::AlignTop | Qt::AlignLeft, e->getText().simplified()); // does this correctly handle UTF-8 I wonder...?

    if(!e->getDataImage().isNull()) {
        painter->drawImage(QRect(rect.right() - 50, rect.top(), 48, 48), e->getDataImage());
    } else {
        painter->drawText(rect, Qt::AlignTop | Qt::AlignRight, e->getDataText().simplified());
    }
    painter->setPen(option.palette.mid().color());
    font.setPointSizeF(font.pointSizeF() * 0.70);
    painter->setFont(font);
    painter->drawText(rect, Qt::AlignBottom | Qt::AlignLeft, e->getInformativeText().simplified());

    painter->restore();
}

int PluginActionListModel::rowCount(const QModelIndex &) const {
    return plugin_list.size();;
}

QVariant PluginActionListModel::data(const QModelIndex &index, int role) const {
    if(!index.isValid())
        return QVariant();
    if(index.row() >= rowCount() || index.row() < 0) {
        return QVariant();
    }

    if(role == Qt::DisplayRole) {
        return qVariantFromValue(plugin_list.at(index.row()));
    }

    return QVariant();
}

void ResultsWindow::on_tblResults_clicked(QModelIndex index)
{
    PluginAction *plugin = qVariantValue<PluginAction*>(index.data());

    // start a timer whose callback will remove the selection that has just happened in the QTableView
    selectionTimer->stop();

    selectionTimer->start(1000);


    plugin->clickAction(this);
}

void ResultsWindow::selectionTimerExpiry()
{
    qDebug() << "Selection timer expiry";
    selectionTimer->stop(); // we don't want to be called again
    QItemSelectionModel *selection = ui->tblResults->selectionModel();
    selection->clear();
}

//void ResultsWindow::hideMeTimerExpiry()
//{
//    this->close();
//}
