#include <QWidget>
#include <QThread>
#include <QWaitCondition>
#include <QDebug>
#include <QMutex>
#include <zbar.h>
#include <iostream>

#include <QFile>
#include <QImage>

#include "zbardecoderthread.h"
#include "common.h"

using namespace std;

ZBarDecoderThread::ZBarDecoderThread()
{
    this->frame_width = WIDTH; // not really necessary as these won't change during the program
    this->frame_height = HEIGHT;
    this->thread_buffer = (char*)malloc(MAX_VIDEO_WIDTH*MAX_VIDEO_HEIGHT*sizeof(char)*4); // need to account for bpp here, but let's assume 32bit pp is the maximum

    thread_output=0;
    hasFoundBarcode=false;
    doQuit=false;
    isDecoding=false;

    this->scanner = zbar_image_scanner_create();
    qDebug() << DEBUGINFO << "Created scanner";
    /* configure the reader */
    //zbar_image_scanner_set_config(scanner, NULL, ZBAR_CFG_ENABLE, 1);
    //zbar_image_scanner_enable_cache(this->scanner, TRUE);
    qDebug() << DEBUGINFO <<  "Enabled scanner";
    this->image = zbar_image_create();
    qDebug() << DEBUGINFO << "Created image";

}

ZBarDecoderThread::~ZBarDecoderThread()
{
    this->doQuit=true;
    this->thread_run.wakeOne();
}


void ZBarDecoderThread::run()
{
    while(true) {

//        cout << "ZBarDecoderThread::run(): in while loop" << endl;

        // this allows the main program to block us until data is available for us
        //this->wait_mutex.lock();
        this->thread_run.wait(&this->wait_mutex); // wait here until signalled
        //this->wait_mutex.unlock();

//        cout << "ZBarDecoderThread::run(): just gone past wait" << endl;

        // see if we should quit
        if(this->doQuit==true){
            cout << "ZBarDecoderThread::run(): doQuit=true" << endl;
            free(this->thread_buffer);
            return;
        }

        // call our function
        this->wait_mutex.lock(); // lock to make sure this is not set anywhere else
        this->isDecoding=true; // let the world know that we are decoding
        this->hasFoundBarcode = libzbar_analyse_image(); // do some work
        this->isDecoding=false; // let the world know that we are done
        this->wait_mutex.unlock(); // unlock, see above


//        cout << "ZBarDecoderThread::run(): just returned from libzbar_analyse_image(): "<< this->hasFoundBarcode << endl;


    }

}


/* Analyse the image data */
int ZBarDecoderThread::libzbar_analyse_image()
{

//    const char *buf = (char *)this->thread_buffer;
//    unsigned int height = this->frame_height;
//    unsigned int width = this->frame_width;
//    int bpp = this->frame_bpp;

//    QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
//    QImage *qi = new QImage((uchar*)buf, width, height, format); //(buffer, width, height, width, format);
//    qi->save("/home/user/zbarthreadimage.png");

//    QFile file("/home/user/zbarthreadimage.raw");
//    file.open(QIODevice::WriteOnly);
//    file.write(buf, height*width*bpp/8); // 4 to make it larger than we want, write all the data
//    file.close();




    zbar_image_set_format(image, *(int*)frame_fourcc); //*(int*)"RGB3");
    zbar_image_set_size(image, frame_width, frame_height);
    zbar_image_set_data(image, thread_buffer, frame_width * frame_height * frame_bpp/8, NULL); //zbar_image_free_data
    image = zbar_image_convert(image, *(int*)"Y800"); // zbar prefers black and white for analysing

    /* scan the image for barcodes */
    zbar_image_scanner_recycle_image(scanner, image);
    int n = zbar_scan_image(scanner, image);
//    qDebug() << "zbar found" << n << "results";
    if(n > 0) {
        /* extract results */
        const zbar_symbol_t *symbol = zbar_image_first_symbol(image); // only use the first symbol
        zbar_symbol_type_t typ = zbar_symbol_get_type(symbol);
        qDebug() << "ZBarDecoderThread::libzbar_analyse_image(): Extracting symbols";

        const char* temp = zbar_symbol_get_data(symbol);
        if(thread_output){
            free(thread_output);
        }
        thread_output = (char *)malloc(strlen(temp)+1);
        strcpy(thread_output,temp);

        barcode_type = zbar_get_symbol_name(typ);

        qDebug() << QString("ZBarDecoderThread::libzbar_analyse_image(): decoded %1 symbol \"%2\"").arg(barcode_type).arg(thread_output);

        //emit imageAnalysed(QString(zbar_get_symbol_name(typ)), QString((char*)(*output)));


        return true;
    } else {
        if(n < 0) {
            qDebug() << "ZBarDecoderThread::libzbar_analyse_image(): zbar error occured";
        }
        //zbar_image_destroy(image);
        return false; //BARCODE_TYPE_NONE;
    }

}


